module RS232(CLK,RST_B,RS232_RX,RX_DATA,RS232_TX);

/***********RX***************/

input CLK;		
input RST_B;	
input RS232_RX;	
output RS232_TX;
output[7:0] RX_DATA;
//------------------------------------------
reg rs232_rx0,rs232_rx1,rs232_rx2,rs232_rx3;
wire neg_rs232_rx;	
reg[12:0] cnt_rx;	
reg clk_bps_rx;
reg bps_start_rx;
reg bps_start_tx;
//******************************************
always @ (posedge CLK or negedge RST_B)
	if(!RST_B) 
	  cnt_rx<=13'd0;
	else if((cnt_rx==2500)||!bps_start_rx)
	  cnt_rx<=13'd0;
	else 
	  cnt_rx <= cnt_rx+1'b1;
//------------------------------------------
always @ (posedge CLK or negedge RST_B)
	if(!RST_B) 
	  clk_bps_rx<=1'b0;
	else if(cnt_rx==1250) 
	  clk_bps_rx<=1'b1;
	else 
	  clk_bps_rx <= 1'b0;
//******************************************
always @ (posedge CLK or negedge RST_B) 
begin
	if(!RST_B) 
		begin
			rs232_rx0 <= 1'b0;
			rs232_rx1 <= 1'b0;
			rs232_rx2 <= 1'b0;
			rs232_rx3 <= 1'b0;
		end
	else begin
			rs232_rx0 <= RS232_RX;
			rs232_rx1 <= rs232_rx0;
			rs232_rx2 <= rs232_rx1;
			rs232_rx3 <= rs232_rx2;
		end
end

assign neg_rs232_rx=rs232_rx3&rs232_rx2&~rs232_rx1&~rs232_rx0;	
//----------------------------------------------------------------

reg[3:0] num_rx;	
reg rx_int;		
//******************************************
always @ (posedge CLK or negedge RST_B)
	if(!RST_B) 
		begin
			bps_start_rx <= 1'bz;
			rx_int <= 1'b0;
		end
	else if(neg_rs232_rx) 
		begin		
			bps_start_rx <= 1'b1;	
			rx_int <= 1'b1;		
		end
	else if(num_rx==4'd12) 
		begin		
			bps_start_rx <= 1'b0;	
			rx_int <= 1'b0;			
		end
//------------------------------------
reg[7:0] rx_data_rx;		
//------------------------------------
reg[7:0] rx_temp_data;	
//------------------------------------
always @ (posedge CLK or negedge RST_B)
	if(!RST_B) 
		begin
			rx_temp_data <= 8'd0;
			num_rx <= 4'd0;
			rx_data_rx <= 8'd0;
		end
	else if(rx_int) 
		begin	
		if(clk_bps_rx) 
			begin	
				num_rx <= num_rx+1'b1;
				case (num_rx)
						4'd1: rx_temp_data[0]<=RS232_RX;	
						4'd2: rx_temp_data[1]<=RS232_RX;	
						4'd3: rx_temp_data[2]<=RS232_RX;	
						4'd4: rx_temp_data[3]<=RS232_RX;	
						4'd5: rx_temp_data[4]<=RS232_RX;	
						4'd6: rx_temp_data[5]<=RS232_RX;	
						4'd7: rx_temp_data[6]<=RS232_RX;	
						4'd8: rx_temp_data[7]<=RS232_RX;	
						default: ;
				endcase
			end
		else if(num_rx == 4'd12) 
			begin		
				num_rx <= 4'd0;			
				rx_data_rx <= rx_temp_data;	
			end
		end

assign RX_DATA = rx_data_rx;	

/*********TX************/

reg[12:0] cnt_tx;
reg clk_bps_tx;
//************************************
always@(posedge CLK or negedge RST_B)
  if(!RST_B) 
    cnt_tx<=13'd0;
  else if((cnt_tx==2500)||!bps_start_tx)
    cnt_tx<=13'd0;
  else 
    cnt_tx<=cnt_tx+1'b1;
//------------------------------------   
always@(posedge CLK or negedge RST_B)
  if(!RST_B) 
    clk_bps_tx<=1'b0;
  else if(cnt_tx==1250)
    clk_bps_tx<=1'b1;
  else 
    clk_bps_tx<=1'b0; 
/***************************************/
reg rx_int0,rx_int1,rx_int2;
wire neg_rx_int;
//--------------------------------------
always@(posedge CLK or negedge RST_B) 
begin
  if(!RST_B) 
    begin
    rx_int0<=1'b0;
    rx_int1<=1'b0;
    rx_int2<=1'b0;
    end
  else 
    begin
    rx_int0<=rx_int;
    rx_int1<=rx_int0;
    rx_int2<=rx_int1;
    end
end    
assign neg_rx_int=~rx_int1&rx_int2;
//--------------------------------------

reg tx_en;
reg[3:0] num_tx;
//**************************************
always@(posedge CLK or negedge RST_B) 
begin
  if(!RST_B) 
    begin
    bps_start_tx<=1'bz;
    tx_en<=1'b0;
    end
  else if(neg_rx_int) 
    begin
    bps_start_tx<=1'b1;
    tx_en<=1'b1;
    end
  else if(num_tx==4'd11) 
    begin
    bps_start_tx<=1'b0;
    tx_en<=1'b0;
    end
end    
//************************************
reg rs232_tx_r;
//------------------------------------
always@(posedge CLK or negedge RST_B) 
begin
  if(!RST_B) 
    begin
    num_tx<=4'd0;
    rs232_tx_r<=1'b1;
    end
  else if(tx_en) 
    begin
        if(clk_bps_tx)
          begin
		  num_tx<=num_tx+1'b1;
		  case(num_tx)
		  4'd0:rs232_tx_r<=1'b0;
		  4'd1:rs232_tx_r<=rx_data_rx[0];
		  4'd2:rs232_tx_r<=rx_data_rx[1];
		  4'd3:rs232_tx_r<=rx_data_rx[2];
		  4'd4:rs232_tx_r<=rx_data_rx[3];
		  4'd5:rs232_tx_r<=rx_data_rx[4];
		  4'd6:rs232_tx_r<=rx_data_rx[5];
		  4'd7:rs232_tx_r<=rx_data_rx[6];
		  4'd8:rs232_tx_r<=rx_data_rx[7];
		  4'd9:rs232_tx_r<=1'b1;
		  default:rs232_tx_r<=1'b1;
		  endcase  
          end
        else if(num_tx==4'd11) num_tx<=4'd0;
    end
end 

assign RS232_TX=rs232_tx_r; 

endmodule
